package applets;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.image.BufferedImage;

//Library imports
import fractals.MandelbrotSet;
import fractals.MandelbrotSetParameters;
import util.color.ColorGradient;
import gui.color.ColorGradientPanel;
import gui.color.IColorGradientPanelListener;
import gui.fractals.MandelbrotSetPanel;

//static imports
import static fractals.MandelbrotSetParameters.*;

/**
 *  Applet for showing the Mandelbrot set.
 *
 **/

public class MandelbrotApplet extends JApplet implements ChangeListener, IColorGradientPanelListener {

  /*** member variables ***/
  ColorGradientPanel m_pColorGradientPanel = null;
  int m_iIterationDepth = 255;
  int m_iLanguageId;
  MandelbrotSetParameters m_pParameters = null;
  MandelbrotSet m_pMandelbrotSet = null;  
  MandelbrotSetPanel m_pMandelbrotSetPanel = null;

  /*** label strings: id 0 = English, id 1 = German ***/
  String[] m_pCol2PosText = {"Color tuning:", "Farbanpassung:"};
  String[] m_pColor1Text = {"Color 1:", "Farbe 1:"};
  String[] m_pColor2Text = {"Color 2:", "Farbe 2:"};
  String[] m_pColorButtonText = {"Change", "Ändern"};
  String[] m_pColorGradientText = {"Color gradient:", "Farbverlauf:"};

  //Called when this applet is loaded into the browser. Creates the GUI.
  public void init() {
    //Execute a job on the event-dispatching thread; creating this applet's GUI.
    try {

      //1. fetch the language parameter and set the language id      
      m_iLanguageId = 0;
      try {
        String pLanguage = getParameter("language");
        if (pLanguage != null) {
          pLanguage = pLanguage.trim().toLowerCase();
          if (pLanguage.equals("german")) {
            m_iLanguageId = 1;
          }
        }
      }
      catch (Exception pException) {
        //nothing to do, no parameter found
      }

      //2. create the gui
      
      //2.1. color gradient panel
      JPanel pControlPanel = new JPanel();
      pControlPanel.setLayout(new GridLayout(2,1));
      
      JLabel pColorGradientLabel = new JLabel(m_pColorGradientText[m_iLanguageId]);
      pControlPanel.add(pColorGradientLabel);
      
      m_pColorGradientPanel = new ColorGradientPanel();
      m_pColorGradientPanel.addColorGradientPanelListener(this);
      m_pColorGradientPanel.setPreferredSize(new Dimension(400, 40));
      pControlPanel.add(m_pColorGradientPanel);
      
      
      
          
      //2.2. create the drawing canvas and fill it with the initial image
      try {
        ColorGradient pColorGradient = new ColorGradient();
        //double dPosition = (double)m_pCol2PosSlider.getValue()/255.0;
        double dPosition = 20.0/255.0;
        pColorGradient.setPositionOfColorPoint(1, dPosition); 
        
        m_pColorGradientPanel.setColorGradient(pColorGradient);
        
        m_pParameters = new MandelbrotSetParameters();
        m_pParameters.setIntegerParameterValue(PARAM_INT_WIDTH, 400);
        m_pParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, 400);
        m_pParameters.setIntegerParameterValue(PARAM_INT_ITERATION_DEPTH, m_iIterationDepth);
        m_pParameters.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, pColorGradient);
        
        m_pMandelbrotSetPanel = new MandelbrotSetPanel();
        m_pMandelbrotSetPanel.setMandelbrotSetParameters(m_pParameters);
        m_pMandelbrotSetPanel.repaint();
       
      }
      catch (Exception pException) {
        //m_pCanvas = new JLabel("The image could not be created! " + pException.getMessage());
      }
          
      //2.3. set the overall layout and add the components
      setLayout(new BorderLayout());
      add(m_pMandelbrotSetPanel, BorderLayout.CENTER);
      add(pControlPanel, BorderLayout.SOUTH);
      
    } catch (Exception e) {
      System.err.println("createGUI didn't complete successfully");
    }
    
    showStatus("started!!! ");
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  /** for sliders. at the moment not used !*/
  public void stateChanged(ChangeEvent e) {

    /*
    //1. set the color tuning parameter
    if (m_pColorGradient == null) m_pColorGradient = new ColorGradient();
    double dPosition = (double)m_pCol2PosSlider.getValue()/255.0;
    m_pColorGradient.setPositionOfColorPoint(1, dPosition); 
    
    m_pColorGradientPanel.setColorGradient(m_pColorGradient);
    
    //2. recreate canvas
    recreateCanvas();
    */
  }
  
  
  
  
  //////////////////////////////////////////
  // IColorGradientPanelListener METHODS  //
  //////////////////////////////////////////
  public void colorGradientChanged(ColorGradientPanel pPanel) {
    
    MandelbrotSetParameters pParameters = m_pMandelbrotSetPanel.getMandelbrotSetParameters();
    pParameters.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, pPanel.getColorGradient());
    m_pMandelbrotSetPanel.setMandelbrotSetParameters(pParameters);
    m_pMandelbrotSetPanel.repaint();
  }
  
  
  ///////////////////////////////////////////
  // HELPER METHODS                        //
  ///////////////////////////////////////////

  
  
  public String createStatusMessage() {
    String pStatusMessage = "";
    /*
    pStatusMessage += m_pCol2PosText[m_iLanguageId] + " ";
    pStatusMessage += m_pCol2PosSlider.getValue();
    
    if (m_pColorGradient == null) m_pColorGradient = new ColorGradient();
    Color pColor1 = m_pColorGradient.getColorOfColorPoint(0);
    pStatusMessage += " - ";
    pStatusMessage += m_pColor1Text[m_iLanguageId] + " ";
    pStatusMessage += pColor1.getRed() + ", " + pColor1.getGreen() + ", " + pColor1.getBlue();
    
    Color pColor2 = m_pColorGradient.getColorOfColorPoint(1);
    pStatusMessage += " - ";
    pStatusMessage += m_pColor2Text[m_iLanguageId] + " ";
    pStatusMessage += pColor2.getRed() + ", " + pColor2.getGreen() + ", " + pColor2.getBlue();
    */
    return pStatusMessage;
  }

  
  
 
  
  
  



}
