package applications;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.util.Hashtable;

/**
 * An entity that holds the default language properties, for example 
 * all label names in English. 
 **/
public class DefaultLanguageProperties {
  
  /** member variables **/
  private Hashtable<String, String> m_pProperties;
  
  /////////////////////////////////
  // CONSTRUCTOR                 //
  /////////////////////////////////
  /**
   *  Creates a language property object holding the default label name settings in English.
   */
  public DefaultLanguageProperties() {
    m_pProperties = new Hashtable<String, String>();
    
    //global names
    m_pProperties.put("AppName", "JFXLibrary Gui Application");
    m_pProperties.put("OK", "OK");
    m_pProperties.put("Abort", "Abort");
    

    //file menu
    m_pProperties.put("FileMenu", "File");
    m_pProperties.put("SaveImage", "Save image");
    m_pProperties.put("Exit", "Exit");
    
    //view menu
    m_pProperties.put("ViewMenu", "View");
    m_pProperties.put("ViewPropertiesFrame", "Show properties frame");

    //image effect menu
    m_pProperties.put("ImageEffectMenu", "Image effects");

    m_pProperties.put("TransformationMenu", "Transformations");
    m_pProperties.put("FractalsMenu", "Fractals");
    m_pProperties.put("MandelbrotSet", "Mandelbrot set");

    m_pProperties.put("ResizeMenu", "Resize");
    m_pProperties.put("DrawingMenu", "Drawing");
    m_pProperties.put("Stars", "Stars");
    m_pProperties.put("Network", "Network");
    m_pProperties.put("BrickTexture", "Brick texture");
    
    m_pProperties.put("ColorMenu", "Color");
    m_pProperties.put("GrayscaleReduction", "Grayscale reduction");
    m_pProperties.put("ColorTransition", "Color transition");
    m_pProperties.put("NoiseGenerator", "Noise generator");
    
    //Configuration menu
    m_pProperties.put("ConfigurationMenu", "Configuration");
    m_pProperties.put("Settings", "Settings");
    
    
    ////////////////////
    //SETTINGS DIALOG
    m_pProperties.put("Settings.Language", "Language:");
    
    ////////////////////
    //PROPERTIES FRAME
    m_pProperties.put("PropertyFrame.Title", "Effect properties");
    
    ////////////////////
    //ERRORS
    m_pProperties.put("Error", "Error");
    m_pProperties.put("Error.NoImageToSave", "There is no image that can be saved!");
    
    ////////////////////
    //IMAGE RESIZE
    m_pProperties.put("ImageResize.Image", "Image");
    m_pProperties.put("ImageResize.MaximumSide", "Maximum side");
    
    ///////////////////
    //STARS
    m_pProperties.put("Stars.PikeNum", "Number of pikes");
    m_pProperties.put("Stars.PikeLength", "Length of pikes");
    m_pProperties.put("Stars.LineNum", "Number of lines");
    m_pProperties.put("Stars.LineColor", "Line color");
    m_pProperties.put("Stars.BackgroundColor", "Background color");
    m_pProperties.put("Stars.HorTrans", "Horizontal translation");
    m_pProperties.put("Stars.VerTrans", "Vertical translation");
    m_pProperties.put("Stars.Rotation", "Rotation");
    
    ///////////////////
    //NETWORK
    m_pProperties.put("Network.PointNum", "Number of points");
    m_pProperties.put("Network.ConnectionNum", "Maximum number of connections");
    m_pProperties.put("Network.PointSize", "Point size");
    m_pProperties.put("Network.BackgroundColor", "Background color");
    m_pProperties.put("Network.PointColor", "Point colors");
    m_pProperties.put("Network.LineColor", "Line color");
    m_pProperties.put("Network.Size", "Size");
    m_pProperties.put("Network.PointIntend", "Point intend");
   
    
    
    
    ///////////////////
    //BRICK TEXTURE
    m_pProperties.put("BrickTexture.BrickColor", "Brick color");
    m_pProperties.put("BrickTexture.BrickBorderColor", "Brick border color");
    m_pProperties.put("BrickTexture.DrawBorder", "Draw brick border");
    m_pProperties.put("BrickTexture.BevelMode", "Bevel mode");
    m_pProperties.put("BrickTexture.RandomMode", "Random mode");
    m_pProperties.put("BrickTexture.BrickColorRandomMagnitude", "Randomize magnitude");
    m_pProperties.put("BrickTexture.BrickWidth", "Brick width");
    m_pProperties.put("BrickTexture.BrickHeight", "Brick height");
    
    m_pProperties.put("BrickTexture.BevelMode.None", "None");
    m_pProperties.put("BrickTexture.BevelMode.Raised", "Raised");
    m_pProperties.put("BrickTexture.BevelMode.Sunk", "Sunk");
    
    m_pProperties.put("BrickTexture.RandomMode.RGB", "RGB");
    m_pProperties.put("BrickTexture.RandomMode.Red", "Red");
    m_pProperties.put("BrickTexture.RandomMode.Green", "Green");
    m_pProperties.put("BrickTexture.RandomMode.Blue", "Blue");
    m_pProperties.put("BrickTexture.RandomMode.RG", "Red - Green");
    m_pProperties.put("BrickTexture.RandomMode.RB", "Red - Blue");
    m_pProperties.put("BrickTexture.RandomMode.GB", "Green - Blue");
    m_pProperties.put("BrickTexture.RandomMode.HSB", "HSB");
    m_pProperties.put("BrickTexture.RandomMode.Hue", "Hue");
    m_pProperties.put("BrickTexture.RandomMode.Saturation", "Saturation");
    m_pProperties.put("BrickTexture.RandomMode.Brightness", "Brightness");
    m_pProperties.put("BrickTexture.RandomMode.HS", "Hue - Saturation");
    m_pProperties.put("BrickTexture.RandomMode.HB", "Hue - Brightness");
    m_pProperties.put("BrickTexture.RandomMode.SB", "Saturation - Brightness");
    
    
    
    ///////////////////
    //MandelbrotSet
    m_pProperties.put("MandelbrotSet.IterationDepth", "Iteration depth");
    m_pProperties.put("MandelbrotSet.ColorGradient", "Color gradient");
    m_pProperties.put("MandelbrotSet.Size", "Size");
    
    ///////////////////
    //GrayscaleReduction
    m_pProperties.put("GrayscaleReduction.Image", "Image");
    m_pProperties.put("GrayscaleReduction.GrayscaleNum", "Number of grayscales");
    
    
    ///////////////////
    //ColorTransition
    m_pProperties.put("ColorTransition.Image", "Image");
    m_pProperties.put("ColorTransition.Color", "Color");
    m_pProperties.put("ColorTransition.TransitionLength", "Transition length");
    m_pProperties.put("ColorTransition.Edges", "Edges");
    
    ///////////////////
    //NoiseGenerator
    m_pProperties.put("NoiseGenerator.Image", "Image");
    m_pProperties.put("NoiseGenerator.NoiseTypes", "Noise types");
    m_pProperties.put("NoiseGenerator.SaltAndPepperIntensity", "Salt and pepper intensity in %");
    m_pProperties.put("NoiseGenerator.GaussianIntensity", "Gaussian noise intensity in %");
    m_pProperties.put("NoiseGenerator.GaussianStandardDeviation", "Gaussian noise standard deviation");
    m_pProperties.put("NoiseGenerator.RandomColorIntensity", "Random color noise intensity in %");
    m_pProperties.put("NoiseGenerator.UseSaltAndPepperNoise", "Salt and Pepper noise");
    m_pProperties.put("NoiseGenerator.UseGaussianNoise", "Gaussian noise");
    m_pProperties.put("NoiseGenerator.UseRandomColorNoise", "Random color noise");

    //////////////////
    //SizingPanel
    m_pProperties.put("SizingPanel.Width", "Width:");
    m_pProperties.put("SizingPanel.Height", "Height:"); 
    
    //////////////////
    //ImageLoadPanel
    m_pProperties.put("ImageLoadPanel.Label", "Load new image");
    m_pProperties.put("ImageLoadPanel.NoFileLabel", "No file loaded!");
    
     //////////////////
    //ZoomPanel
    m_pProperties.put("ZoomPanel.FitWidth", "Fit width");
    m_pProperties.put("ZoomPanel.FitHeight", "Fit height");
    
  }
  
  
  //////////////////////////////////
  // GET METHODS                  //
  //////////////////////////////////
  
  /**
   *  Returns the value of the specified property.
   *
   *  @param pProperty A string indicating the name of the property.
   *
   *  @return The value of the specified property as string.
   **/
  public String getProperty(String pProperty) {
    //1. make an initial check
    if (pProperty == null) return null;
    if (pProperty.trim().equals("")) return null;
    
    //2. return the value
    return m_pProperties.get(pProperty);
  }
}