package applications;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//library imports
import standard.VideoEffect;
import standard.ImageEffect;
import standard.ImageSequence;
import io.*;
import io.imageeffects.*;


// not necessary standard imports
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;


/**
 *  The command line application.
 **/

public class FXLibraryCmd {

  private static final String[] VIDEOEFFECTS = {"FrameDifferencing"};
  private static final String[] IMAGEEFFECTS = {"Stars"};


  /**
   *  Starts the command line application.
   *
   *  @param args The string array that is used to pass parameters to the command line application.
   */
  public static void main(String[] args) {
  
    //1. check the input array
    if ((args == null) || (args.length != 2)) {
      printUsage();
      return;
    }
  
    //2. check the effect name
    String pEffectName = args[0].trim();
    int iEffectNum = VIDEOEFFECTS.length;
    boolean bVideoEffect = false;
    
    //2.1. video effects
    for (int i = 0; i < iEffectNum; i++) {
      if (pEffectName.equalsIgnoreCase(VIDEOEFFECTS[i])) {
        bVideoEffect = true;
      }
    }
    
    //2.2. image effects
    iEffectNum = IMAGEEFFECTS.length;
    boolean bImageEffect = false;
    for (int i = 0; i < iEffectNum; i++) {
      if (pEffectName.equalsIgnoreCase(IMAGEEFFECTS[i])) {
        bImageEffect = true;
      }
    }
    
    //2.3. finish if the effect could not be found
    if ((!bVideoEffect) && (!bImageEffect)) {
      System.out.println("The effect '" + pEffectName + "' is not implemented!");
      System.out.println("");
      printValidEffects();
    }
    
    
    //3. create the configuration
    Configuration pConfiguration = new Configuration(args[1]);
    
    
    //4. create a new rendering output
    RenderingOutput pRenderOutput = RenderingOutputConfiguration.getRenderingOutput(pConfiguration);
    
    
    
    //5. create and process the effect
    
    //video effect
    if (bVideoEffect) {
      
      //create the effect
      VideoEffect pEffect = null;
      if (pEffectName.equalsIgnoreCase(VIDEOEFFECTS[0])) {
        try {
          pEffect = FrameDifferencingConfiguration.getEffect(pConfiguration);
        }
        catch (Exception pException) {
          System.out.println("The Effect could not be created!");
          pException.printStackTrace();
          return;
        }
      }
      pEffect.addVideoEffectListener(new ProgressOutput());
      pEffect.addVideoEffectListener(pRenderOutput);
    
      //process the effect
      pEffect.process();
      return;
      
    }
    
    if (bImageEffect) {
      //create the effect
      ImageEffect pEffect = null;
      if (pEffectName.equalsIgnoreCase(IMAGEEFFECTS[0])) {
        try {
          pEffect = StarsConfiguration.getEffect(pConfiguration);
        }
        catch (Exception pException) {
          System.out.println("The Effect could not be created!");
          pException.printStackTrace();
          return;
        }
      }
      
      //process the effect
      try {
        BufferedImage pImage = pEffect.process();
        ImageIO.write(pImage, "png", new File("output.png"));
      }
      catch (Exception pException) {
        System.out.println("The effect could not be processed!");
        pException.printStackTrace();
      }
      return;
    }
  
        
  
  }
  
  
  
  
  
  
  /////////////////////////////
  // PRIVATE HELPER METHODS  //
  /////////////////////////////
  private static void printUsage() {
    System.out.println("USAGE:   FXLibraryCmd <effectname> <configfile>");
    System.out.println("");
    System.out.println("");
  }
  
  private static void printValidEffects() {
    
    System.out.println("Valid video effects:");
    System.out.println("");
    
    int iEffectNum = VIDEOEFFECTS.length;    
    for (int i = 0; i < iEffectNum; i++) {
      System.out.println(VIDEOEFFECTS[i]);
    }
    
    System.out.println("");
    System.out.println("");
    
    System.out.println("Valid image effects:");
    System.out.println("");
    
    iEffectNum = VIDEOEFFECTS.length;    
    for (int i = 0; i < iEffectNum; i++) {
      System.out.println(IMAGEEFFECTS[i]);
    }
    
    System.out.println("");
    System.out.println("");
    
  }

}