package applications;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

// oeffnungsort wird nicht uebernommen


//gui imports
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;

import java.util.Vector;

//event imports
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.imageio.ImageIO;
import java.io.File;

//library imports
import standard.ImageSequence;
import gui.AbstractImageEffectPanel;
import gui.AbstractPropertiesPanel;
import gui.SimpleImageEffectPanel;
import gui.fractals.MandelbrotSetPanel;
import gui.drawing.StarsPanel;
import gui.ImageEffectInternalFrame;
import gui.PropertiesInternalFrame;
import color.GrayscaleReduction;
import color.NoiseGenerator;
import color.ColorTransition;
import color.ColorTransitionParameters;
import transformation.ImageResize;
import transformation.ImageResizeParameters;
import drawing.Network;
import drawing.BrickTexture;

/**
 *  The starting point and main frame of the gui application of the JFXLibrary.
 **/

public class GuiApplication extends JFrame implements ActionListener, ISettingsListener {

  /*** member variables ***/
  JDesktopPane m_pDesktop;
  
  LanguageProperties m_pLanguageProperties = null;
  
  /*** gui member variables ***/
  JMenu m_pFileMenu, m_pViewMenu, m_pImageEffectMenu, m_pFractalsMenu;
  JMenu m_pTransformationMenu, m_pDrawingMenu, m_pColorMenu, m_pConfigurationMenu;
  JMenuItem m_pSaveMenu, m_pExitMenu, m_pMandelbrotSet, m_pStars, m_pNetwork, m_pSettingsMenu;
  JMenuItem m_pGrayscaleReduction, m_pColorTransition, m_pBrickTexture, m_pResizeMenu;
  JMenuItem m_pNoiseGenerator;
  JCheckBoxMenuItem m_pViewPropertiesFrame;
  
  Vector<ImageEffectInternalFrame> m_pInternalFrames = null;
  ImageEffectInternalFrame m_pCurrentInternalFrame = null;
  PropertiesInternalFrame m_pPropertiesFrame = null;
  
  SettingsDialog m_pSettingsDialog = null;
  
  
  /*** new gui members ***/
  AbstractImageEffectPanel m_pCurrentImageEffectPanel = null;
  AbstractPropertiesPanel m_pCurrentPropertiesPanel = null;
  
   
 

  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates the main frame of the GUI application.
   **/
  public GuiApplication() {
    
    //1. init the frame
    super();
    
    //2. settings
    try {
      m_pSettingsDialog = new SettingsDialog(this);
      m_pSettingsDialog.addSettingsListener(this);
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }
    
    

    //3. set the size and position of the frame
    //   Make the big window be indented 50 pixels from each edge
    //   of the screen.
    int iInset = 50;
    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iWindowWidth = pScreenSize.width  - iInset*2;
    int iWindowHeight = pScreenSize.height - iInset*2;
    setBounds(iInset, iInset, iWindowWidth, iWindowHeight);
    setExtendedState(MAXIMIZED_BOTH); //set application maximized
    

    //4. Set up the GUI.
    //m_pDesktop = new JDesktopPane(); //a specialized layered pane
    //setContentPane(m_pDesktop);
    setJMenuBar(createMenuBar());
    setLayout(new BorderLayout());
    
    
    
    /*
    //5. create the properties frame
    int iPropFrameWidth = 300;
    int iPropFrameHeight = iWindowHeight - 100;
    m_pPropertiesFrame = new PropertiesInternalFrame(iPropFrameWidth, iPropFrameHeight);
    m_pPropertiesFrame.setLocation(iWindowWidth - iPropFrameWidth, 0);
    try {
      m_pDesktop.add(m_pPropertiesFrame);
      m_pPropertiesFrame.setSelected(true);
      m_pPropertiesFrame.setVisible(true);
    }
    catch (Exception pException) {}
    */
    
    
    //finally apply the settings
    applyNewSettings(SettingsDialog.getSettings());
        
    

    
  }
  
  
  ////////////////////////////////
  // MENU METHODS               //
  ////////////////////////////////
  private JMenuBar createMenuBar() {
    
    
    //1. create the menu bar
    JMenuBar pMenuBar = new JMenuBar();

    
    //2. file menu
    m_pFileMenu = new JMenu();
    m_pSaveMenu = new JMenuItem();
    m_pSaveMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    m_pSaveMenu.addActionListener(this);
    m_pFileMenu.add(m_pSaveMenu);
    
    m_pExitMenu = new JMenuItem();
    m_pExitMenu.addActionListener(this);
    m_pFileMenu.addSeparator();
    m_pFileMenu.add(m_pExitMenu);
    
    //3. view menu
    m_pViewMenu = new JMenu();
    m_pViewPropertiesFrame = new JCheckBoxMenuItem();
    m_pViewPropertiesFrame.setSelected(true);
    m_pViewPropertiesFrame.addActionListener(this);
    m_pViewMenu.add(m_pViewPropertiesFrame);
    
    //4. image effects
    m_pImageEffectMenu = new JMenu();
    
    m_pTransformationMenu = new JMenu();
    m_pResizeMenu = new JMenuItem();
    m_pResizeMenu.addActionListener(this);
    m_pTransformationMenu.add(m_pResizeMenu);
    m_pImageEffectMenu.add(m_pTransformationMenu);
    
    m_pFractalsMenu = new JMenu();
    m_pMandelbrotSet = new JMenuItem();
    m_pMandelbrotSet.addActionListener(this);
    m_pFractalsMenu.add(m_pMandelbrotSet);
    m_pImageEffectMenu.add(m_pFractalsMenu);
    
    m_pDrawingMenu = new JMenu();
    m_pStars = new JMenuItem();
    m_pStars.addActionListener(this);
    m_pDrawingMenu.add(m_pStars);
    
    m_pNetwork = new JMenuItem();
    m_pNetwork.addActionListener(this);
    m_pDrawingMenu.add(m_pNetwork);

    m_pBrickTexture = new JMenuItem();
    m_pBrickTexture.addActionListener(this);
    m_pDrawingMenu.add(m_pBrickTexture);
    m_pImageEffectMenu.add(m_pDrawingMenu);
    
    
    m_pColorMenu = new JMenu();
    m_pGrayscaleReduction = new JMenuItem();
    m_pGrayscaleReduction.addActionListener(this);
    m_pColorMenu.add(m_pGrayscaleReduction);
    
    m_pColorTransition = new JMenuItem();
    m_pColorTransition.addActionListener(this);
    m_pColorMenu.add(m_pColorTransition);
    
    m_pNoiseGenerator = new JMenuItem();
    m_pNoiseGenerator.addActionListener(this);
    m_pColorMenu.add(m_pNoiseGenerator);
    m_pImageEffectMenu.add(m_pColorMenu);
    
    //4. configuration menu
    m_pConfigurationMenu = new JMenu();
    m_pSettingsMenu = new JMenuItem();
    m_pSettingsMenu.addActionListener(this);
    m_pConfigurationMenu.add(m_pSettingsMenu);
    
    //finally add all menus to the menubar
    pMenuBar.add(m_pFileMenu);
    pMenuBar.add(m_pViewMenu);
    pMenuBar.add(m_pImageEffectMenu);
    pMenuBar.add(m_pConfigurationMenu);
    return pMenuBar;

  }
  
  //////////////////////////
  // SET METHODS          //
  //////////////////////////
  /**
   *  Sets the panel of the effect that should be shown now. 
   *
   *  @param pImageEffectInternalFrame The effect panel that should be shown now.
   **/
  public void setCurrentEffectPanel(ImageEffectInternalFrame pImageEffectInternalFrame) {
    if (pImageEffectInternalFrame == null) return;
    m_pCurrentInternalFrame = pImageEffectInternalFrame;
  }
  
  
  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  Implementation of the {@code ActionListener} interface to react on menu actions or changes in the settings.
   *
   *  @param e The {@code ActionEvent} from which the source of the action is fetched.
   **/
  public void actionPerformed(ActionEvent e) {
    String pActionCommand = e.getActionCommand();
    Object pSource = e.getSource();
    
    
    //1. the exit
    if (pSource.equals(m_pExitMenu)) {
      System.exit(0);
    }
    
    
    //2. the options
    if (pSource.equals(m_pSettingsMenu)) {
      
      try {
        m_pSettingsDialog.setVisible(true);
      }
      catch (Exception pException) {
        pException.printStackTrace();
      }
      
      return;
    }
    
    //3. save some images
    if (pSource.equals(m_pSaveMenu)) {
      //I. fetch the frame that has the focus
      boolean bNoImageAvailable = (m_pCurrentImageEffectPanel == null);
      
      if (bNoImageAvailable) {
        //to do: put those messages into the config file
        String pError = m_pLanguageProperties.getProperty("Error.NoImageToSave");
        String pTitle = m_pLanguageProperties.getProperty("Error");
        JOptionPane.showMessageDialog(this, pError, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }
      
      
      
      //II. get the image of this frame
      BufferedImage pImage = null;
      try {
        pImage = m_pCurrentImageEffectPanel.getEffectImage();
      }
      catch (Exception pException) {
        String pError = pException.getMessage();
        String pTitle = m_pLanguageProperties.getProperty("Error");
        JOptionPane.showMessageDialog(this, pError, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }
      
      //III. save the image 
      JFileChooser pChooser = new JFileChooser();
      FileNameExtensionFilter pFilter = new FileNameExtensionFilter("Image files", "png", "bmp", "gif", "jpg", "jpeg");
      pChooser.setFileFilter(pFilter);
      File pCurrentDirectory = null;
      Settings pSettings = SettingsDialog.getSettings();
      try {
        pCurrentDirectory = new File(pSettings.getProperty("StoreImage.directory"));
      }
      catch (Exception pException) {}
      pChooser.setCurrentDirectory(pCurrentDirectory);
      
      String pCurrentFileName = null;
      if (m_pCurrentPropertiesPanel != null) pCurrentFileName = m_pCurrentPropertiesPanel.getFileName();
      if (pCurrentFileName != null) pChooser.setSelectedFile(new File(pCurrentFileName));
      
      try {
        int iResult = pChooser.showSaveDialog(this);
        if (iResult == JFileChooser.APPROVE_OPTION) {
          File pFile = pChooser.getSelectedFile();
          ImageSequence.saveImage(pImage, pFile);
          
          pSettings.setProperty("StoreImage.directory", pChooser.getCurrentDirectory().getAbsolutePath());
          pSettings.store(SettingsDialog.SETTINGS_PROPERTY_FILE);
        }
      }
      catch (Exception pException) {
        pException.printStackTrace();
      }
      return;
    }
    
    
    //4. show properties frame
    if (pSource.equals(m_pViewPropertiesFrame)) {
      //m_pPropertiesFrame.setVisible(m_pViewPropertiesFrame.isSelected());
      if (m_pCurrentPropertiesPanel != null) m_pCurrentPropertiesPanel.setVisible(m_pViewPropertiesFrame.isSelected());
      return;
    }
    
    //5. create an effect image, because this is the fun, it is about
    AbstractImageEffectPanel pPanel = null;
    if (pSource.equals(m_pMandelbrotSet)) {
      pPanel = new MandelbrotSetPanel();
    }
    else if (pSource.equals(m_pStars)) {
      pPanel = new StarsPanel();
    }
    else if (pSource.equals(m_pNetwork)) {
      pPanel = new SimpleImageEffectPanel(new Network());
    }
    else if (pSource.equals(m_pBrickTexture)) {
      pPanel = new SimpleImageEffectPanel(new BrickTexture());
    }
    else if (pSource.equals(m_pGrayscaleReduction)) {
      pPanel = new SimpleImageEffectPanel(new GrayscaleReduction());
    }
    else if (pSource.equals(m_pColorTransition)) {
      //to do load an image -> FileLoadPanel = only for the effect properties panel
      ColorTransitionParameters pParameters = new ColorTransitionParameters();
      ColorTransition pColorTransition = new ColorTransition(pParameters);
      pPanel = new SimpleImageEffectPanel(pColorTransition);
    }
    else if (pSource.equals(m_pNoiseGenerator)) {
      pPanel = new SimpleImageEffectPanel(new NoiseGenerator());
    }
    else if (pSource.equals(m_pResizeMenu)) {
      pPanel = new SimpleImageEffectPanel(new ImageResize());
    }
    
    
    setImageEffectPanel(pActionCommand, pPanel);
  }
  
  
  ////////////////////////////////////
  // ISettingsListener METHODS      //
  ////////////////////////////////////
  
  /**
   *  Apply the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    m_pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    
    //3. set the labels for the application
    setTitle(m_pLanguageProperties.getProperty("AppName"));
    
    //4. set all menu labels
    m_pFileMenu.setText(m_pLanguageProperties.getProperty("FileMenu"));
    m_pSaveMenu.setText(m_pLanguageProperties.getProperty("SaveImage"));
    m_pExitMenu.setText(m_pLanguageProperties.getProperty("Exit"));
    
    m_pViewMenu.setText(m_pLanguageProperties.getProperty("ViewMenu"));
    m_pViewPropertiesFrame.setText(m_pLanguageProperties.getProperty("ViewPropertiesFrame"));

    
    m_pImageEffectMenu.setText(m_pLanguageProperties.getProperty("ImageEffectMenu"));
    
    m_pTransformationMenu.setText(m_pLanguageProperties.getProperty("TransformationMenu"));
    m_pResizeMenu.setText(m_pLanguageProperties.getProperty("ResizeMenu"));
    
    m_pFractalsMenu.setText(m_pLanguageProperties.getProperty("FractalsMenu"));
    m_pMandelbrotSet.setText(m_pLanguageProperties.getProperty("MandelbrotSet"));
    
    
    m_pDrawingMenu.setText(m_pLanguageProperties.getProperty("DrawingMenu"));
    m_pStars.setText(m_pLanguageProperties.getProperty("Stars"));
    m_pNetwork.setText(m_pLanguageProperties.getProperty("Network"));
    m_pBrickTexture.setText(m_pLanguageProperties.getProperty("BrickTexture"));
    
    m_pColorMenu.setText(m_pLanguageProperties.getProperty("ColorMenu"));
    m_pGrayscaleReduction.setText(m_pLanguageProperties.getProperty("GrayscaleReduction"));
    m_pColorTransition.setText(m_pLanguageProperties.getProperty("ColorTransition"));
    m_pNoiseGenerator.setText(m_pLanguageProperties.getProperty("NoiseGenerator"));
    
    m_pConfigurationMenu.setText(m_pLanguageProperties.getProperty("ConfigurationMenu"));
    m_pSettingsMenu.setText(m_pLanguageProperties.getProperty("Settings"));
    
  }
  
  
  private void setImageEffectPanel(String pTitle, AbstractImageEffectPanel pPanel) {
    //1. make some checks
    if (pPanel == null) return;
    if (pTitle == null) pTitle = "";
    pTitle = pTitle.trim();
    
    //2. remove the old image effect panel
    if (m_pCurrentImageEffectPanel != null) {
      remove(m_pCurrentImageEffectPanel);
    }
    
    //3. set the new image effect panel
    add(pPanel, BorderLayout.CENTER);
    m_pCurrentImageEffectPanel = pPanel;
    pPanel.revalidate();
    
    //4. set the title
    setTitle(m_pLanguageProperties.getProperty("AppName") + " - " + pTitle);
    
    //5. set the properties panel
    setPropertiesPanel(m_pCurrentImageEffectPanel.getPropertiesPanel());
    
    m_pSettingsDialog.addSettingsListener(pPanel);
    
  }
  
  
  /**
   *  Sets the properties panel that should be shown now to steer the parameters of the current effect. 
   *
   *  @param pPropertiesPanel The effect properties panel that should be shown to steer the effect parameters.
   **/
  public void setPropertiesPanel(AbstractPropertiesPanel pPropertiesPanel) {
    //1. make a simple check
    if (pPropertiesPanel == null) return;
    
    //2. remove the old panel
    if (m_pCurrentPropertiesPanel != null) {
      remove(m_pCurrentPropertiesPanel);
    }
    
    //3. set the new panel
    m_pCurrentPropertiesPanel = pPropertiesPanel;
    add(m_pCurrentPropertiesPanel, BorderLayout.SOUTH);
    m_pCurrentPropertiesPanel.setVisible(m_pViewPropertiesFrame.isSelected());
    m_pCurrentPropertiesPanel.revalidate(); //is needed to show the new panel immediately.
    
    //4. add the panel to the setting listener
    m_pSettingsDialog.addSettingsListener(m_pCurrentPropertiesPanel);
    
    
      
  }
  
  
  
  private void createFrame(String pTitle, AbstractImageEffectPanel pPanel) {
  
    //0. check if there is a panel
    if (pPanel == null) return;
    
    //1. check if the data structure has to be initiated
    if (m_pInternalFrames == null) m_pInternalFrames = new Vector<ImageEffectInternalFrame>();
  
    //2. create the internal frame
    ImageEffectInternalFrame pFrame = new ImageEffectInternalFrame(pTitle, pPanel, m_pPropertiesFrame, this);
    pFrame.setSize(new Dimension(getWidth() - m_pPropertiesFrame.getSize().width, getHeight() - 100));
    pFrame.setLocation(0, 0);
    pFrame.setVisible(true);
    
    
    //3. show the internal frame and set the image effect picture
    try {
      m_pDesktop.add(pFrame);
      pFrame.setSelected(true);
    }
    catch (Exception pException) {}
    
    
    
    //4. add the frame to the data structure
    m_pInternalFrames.add(pFrame);
  }
  
  
  ////////////////////////////////
  // START METHOD               //
  ////////////////////////////////
  
  /** 
   *  Starts the GUI application. 
   *
   *  @param args A string array to pass on parameters. But it is not used at the moment.
  **/
  public static void main(String[] args) {
  
    //Make sure we have nice window decorations.
    try {
	    // Set System L&F
        UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
    } 
    catch (Exception e) {
       // handle exception
    }

    //JFrame.setDefaultLookAndFeelDecorated(true);
    

    //Create and set up the window.
    GuiApplication pFrame = new GuiApplication();
    pFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    //Display the window.
    pFrame.setVisible(true);
      
  }
  
  
  
  
  
  
 
}