package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JLabel;

/**
 *  A collection of methods that helps to organize components in a grid bag layout, <b>which is no longer used!!!</b>
 **/
public class GridBagLayoutUtilities {



  /**
   *  Configures and returns the {@code GridBagConstraints} object so that it can be used in property panels <b>(No longer used!!!)</b>. 
   *
   *  @param pGridBagConstraints The {@code GridBagConstraints} object that should be configured for the default use in property panels.
   *
   *  @return The {@code GridBagConstraints} object that has been configured for the default use in property panels.
   **/
  public static GridBagConstraints getDefaultConstraintsForPropertyPanels(GridBagConstraints pGridBagConstraints) {
    //1. make a simple check
    if (pGridBagConstraints == null) pGridBagConstraints = new GridBagConstraints();
    
    //2. make the settings
    pGridBagConstraints.gridwidth = 1; //number of columns
    pGridBagConstraints.gridx = 0; //the first x cell coordinate
    pGridBagConstraints.anchor = GridBagConstraints.CENTER; //position of the element within its cell
    pGridBagConstraints.fill = GridBagConstraints.NONE; //the mode how the element fills out its cell - NONE the preferred size
    pGridBagConstraints.insets = new Insets(10, 5, 10, 5); // the minimum gaps between grid cells (top, left, bottom, right)
    pGridBagConstraints.weighty = 0; // additional space by vertical resizing is NOT used to increase the cells
    
    //3. return the constraints
    return pGridBagConstraints;
  }
  
  
  
  /**
   *  Adds a buffer component to the current line of the {@code GridBagLayout} to create a gap between two elements <b>(No longer used!!!)</b>. 
   *
   *  @param pGridBagConstraints The {@code GridBagConstraints} object that is used to add this buffer element.
   *  @param pPanel The {@code Panel} object that shows the {@code GridBagLayout}.
   *
   **/
  public static void addDefaultBufferComponentForPropertyPanels(
    GridBagConstraints pGridBagConstraints, 
    JPanel pPanel) {
    
    //1. make some checks
    if (pPanel == null) return;
    if (pGridBagConstraints == null) return;
    
    //2. create a buffer object that takes all the space at the bottom, so that the stays stay on top
    JLabel pBufferLabel = new JLabel();
    pGridBagConstraints.gridy++;
    pGridBagConstraints.fill = GridBagConstraints.BOTH; // the element fully fills the grid cell 
    pGridBagConstraints.weighty = 1; // additional space by vertical resizing is now fully used to increase the cell of this element
    pPanel.add(pBufferLabel, pGridBagConstraints);
    
  }
    
  

}