package gui.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import color.GrayscaleReduction;
import color.GrayscaleReductionParameters;
import gui.AbstractPropertiesPanel;
import gui.ISlimSliderPanelListener;
import gui.SlimSliderPanel;

//import gui.io.FileLoadPanel;
//import gui.io.IFileLoadPanelListener;

import gui.ImageLoadPanel;
import gui.IImageLoadPanelListener;

import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static color.GrayscaleReductionParameters.*;

/**
 *  A properties panel for the gray scale reduction effect.
 **/
public class GrayscaleReductionPropertiesPanel 
  extends AbstractPropertiesPanel 
  implements ISlimSliderPanelListener, IImageLoadPanelListener, ActionListener {
  
  /*** member variables ***/
  private GrayscaleReductionParameters m_pParameters;

  /*** gui member variables ***/
  //private FileLoadPanel m_pFileLoadPanel;
  private ImageLoadPanel m_pImageLoadPanel;
  private SlimSliderPanel m_pGrayscaleNumSliderPanel;
  
  private JPanel m_pCurrentPropertyPanel;
  
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code GrayscaleReductionPropertiesPanel} object with the default parameters 
   *  of the {@code GrayscaleReduction} effect.
   **/
  public GrayscaleReductionPropertiesPanel() {
    //1. init members
    m_pParameters = null;
  
    //2. set the layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    
    //3. add the properties selection
    
    
    
    
    
    
    //4. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addImageLoadPanelListener(this);
    m_pCurrentPropertyPanel = m_pImageLoadPanel;
    
    
    /*
    m_pFileLoadPanel = new FileLoadPanel();
    m_pFileLoadPanel.setText("Load image");
    m_pFileLoadPanel.addFileLoadPanelListener(this);
    add(m_pFileLoadPanel, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate
    */
    
    
    //5. create the panel for the grayscale num
    m_pGrayscaleNumSliderPanel = new SlimSliderPanel(1, 64, 8);
    m_pGrayscaleNumSliderPanel.addSlimSliderPanelListener(this);
    
   
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
        
    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties);
  }
  
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code GrayScaleReductionParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code GrayScaleReductionParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code GrayscaleReductionPropertiesPanel} object. 
   *  The set operation is only successful if a {@code GrayscaleReductionParameters} object has been specified.
   *
   *  @param pParameters The {@code GrayscaleReductionParameters} object representing the effect parameters 
   *                     that should be represented by this {@code GrayscaleReductionPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;
    
    if (pParameters instanceof GrayscaleReductionParameters) {
      m_pParameters = (GrayscaleReductionParameters)pParameters;
      
      updateControls();
    }
  }
  
  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the number of gray scales has been changed in the slider panel for this parameter, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SlimSliderPanel} object that steers the number of gray scales.
   *  @param iValue The number of gray scales. 
   **/
  public void applyChanges(SlimSliderPanel pSliderPanel, int iValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pParameters == null) m_pParameters = new GrayscaleReductionParameters();
  
    //2. set the new parameters
    m_pParameters.setIntegerParameterValue(PARAM_INT_GRAYSCALENUM, iValue);
    
    //3. fire the apply changes event
    fireApplyChanges(m_pParameters);

  }
  
  /*
  ///////////////////////////////////
  // IFILE LOAD PANEL LISTENER     //
  ///////////////////////////////////
  public void applyNewImage(BufferedImage pImage) {
    if (m_pParameters == null) m_pParameters = new GrayscaleReductionParameters();
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);
    fireApplyChanges(m_pParameters);
  }
  */
  
  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image to which the effect should be applied is changed, this is communicated with this method, 
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image to which the effect is applied.
   *  @param pImage The image to which the effect should be applied.  
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pParameters == null) m_pParameters = new GrayscaleReductionParameters();
    
    //1. set the new colors
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);
    
    //2. fire the apply changes event
    fireApplyChanges(m_pParameters);
  }
  
  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  This sets the new parameter panel that should be modified. The parameter is chosen by a combo box. 
   *
   *  @param pEvent The action event triggering this method. 
   **/
  public void actionPerformed(ActionEvent pEvent) {
    
    //1. get the selected property
    
    
    //2. remove the current property panel
    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    
    //3. set the new property panel
    int iSelectedId = getSelectedPropertiesIndex();
    if (iSelectedId == 0) {
      m_pCurrentPropertyPanel = m_pImageLoadPanel;
    }
    else if (iSelectedId == 1) {
      m_pCurrentPropertyPanel = m_pGrayscaleNumSliderPanel;
    }
   
    add(m_pCurrentPropertyPanel);
    
    repaint();
    revalidate();
    
    
    
  }
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    populatePropertiesSelection(pLanguageProperties);
    
    
    
    
    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }
    
    
   
    
  }
  
 
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pParameters == null) return;
    
    //2. TO DO: update all controls according to the settings 
    //   in the effect parameters and make sure that no action is fired!
  }
  
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties) {
    
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add("GrayscaleReduction.Image");
    pPropertyKeys.add("GrayscaleReduction.GrayscaleNum");
    populatePropertiesSelection(pLanguageProperties, pPropertyKeys);
    
    
  }
  
  
}