package gui.drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JColorChooser;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import drawing.BrickTextureParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SlimSliderPanel;
import gui.ISlimSliderPanelListener;
import gui.ComboBoxPanel;
import gui.IComboBoxPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static drawing.BrickTextureParameters.*;


/**
 *  A properties panel for the brick texture effect.
 **/
public class BrickTexturePropertiesPanel extends AbstractPropertiesPanel 
  implements ISlimSliderPanelListener, IColorDisplayListener, IComboBoxPanelListener, ActionListener {
  
  /*** member variables ***/
  private BrickTextureParameters m_pBrickTexturePrameters;

  /*** gui member variables ***/
  private SlimSliderPanel m_pBrickColorRandomSliderPanel, m_pBrickWidthSliderPanel, m_pBrickHeightSliderPanel;
  private ColorDisplay m_pBrickColorDisplay, m_pBrickBorderColorDisplay;
  private ComboBoxPanel m_pRandomModes, m_pBevelModes;
  private JCheckBox m_pDrawBrickBorder;
  
  private JPanel m_pBrickBorderPanel;
  
  private JPanel m_pCurrentPropertyPanel;
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code BrickTexturePropertiesPanel} object with the default parameters 
   *  of the {@code BrickTexture} effect.
   **/
  public BrickTexturePropertiesPanel() {
    //0. init members
    m_pBrickTexturePrameters = null;
    
    //1. set the layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    
    /*
    //1. set the layout
    JTabbedPane pTabbedPane = new JTabbedPane();
    pTabbedPane.addTab("Color", getColorDisplay());
    pTabbedPane.addTab("Size", getSizePanel());
    add(pTabbedPane);
    */
    
    
    //3. create the panel
    //3.1. brick color
    m_pBrickColorDisplay = new ColorDisplay();
    m_pBrickColorDisplay.setColor(Color.CYAN);
    m_pBrickColorDisplay.addColorDisplayListener(this);
    
    m_pCurrentPropertyPanel = m_pBrickColorDisplay;
    add(m_pCurrentPropertyPanel);
    
    //3.2. randomize magnitude
    m_pBrickColorRandomSliderPanel = new SlimSliderPanel(0, 500, 150);
    m_pBrickColorRandomSliderPanel.addSlimSliderPanelListener(this);
    
    //3.3. color random mode - TO DO: bundle this into one method
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    
    Vector<String> pRandomModeValues = new Vector<String>();
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RGB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Red"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Green"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Blue"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RG"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.GB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HSB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Hue"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Saturation"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Brightness"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HS"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.SB"));
    m_pRandomModes = new ComboBoxPanel("", pRandomModeValues, 0);
    m_pRandomModes.addComboBoxPanelListener(this);
    
    //3.4. bevel mode
    Vector<String> pBevelModeValues = new Vector<String>();
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.None"));
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.Raised"));
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.Sunk"));
    m_pBevelModes = new ComboBoxPanel("", pBevelModeValues, 1);
    m_pBevelModes.addComboBoxPanelListener(this);
    
    
    //3.5. brick border color
    m_pBrickBorderPanel = new JPanel();
    m_pBrickBorderPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    
    m_pDrawBrickBorder = new JCheckBox(pLanguageProperties.getProperty("BrickTexture.DrawBorder"));
    m_pDrawBrickBorder.setSelected(false);
    m_pDrawBrickBorder.addActionListener(this);
    m_pBrickBorderPanel.add(m_pDrawBrickBorder);
    
    m_pBrickBorderColorDisplay = new ColorDisplay();
    m_pBrickBorderColorDisplay.setColor(Color.BLACK);
    m_pBrickBorderColorDisplay.addColorDisplayListener(this);
    m_pBrickBorderPanel.add(m_pBrickBorderColorDisplay);
    
    //3.6. brick width
    m_pBrickWidthSliderPanel = new SlimSliderPanel(0, 500, 20);
    m_pBrickWidthSliderPanel.addSlimSliderPanelListener(this);
    
    //3.7. brick height
    m_pBrickHeightSliderPanel = new SlimSliderPanel(0, 500, 10);
    m_pBrickHeightSliderPanel.addSlimSliderPanelListener(this);
    
    
    

    
    //finally make new settings
    //applyNewSettings(SettingsDialog.getSettings());
    
    //2. create the combo box with the properties
    
    
    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties);

    
  }
  
  ///////////////////////////
  // PRIVATE GUI METHODS   //
  ///////////////////////////
  /*
  private JPanel getColorDisplay() {
    //1. set the layout
    JPanel pPanel = new JPanel();
    GridBagLayout pGridBag = new GridBagLayout();
    GridBagConstraints pGridBagConstraints = GridBagLayoutUtilities.getDefaultConstraintsForPropertyPanels(null);
    pGridBagConstraints.fill = GridBagConstraints.HORIZONTAL; //the mode how the element fills out its cell
    pPanel.setLayout(pGridBag);
    
    
    //2. create the panel for brick color
    m_pBrickColorDisplay = new ColorDisplay();
    m_pBrickColorDisplay.setColor(Color.CYAN);
    m_pBrickColorDisplay.addColorDisplayListener(this);
    pPanel.add(m_pBrickColorDisplay, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //3. create the panel for the brick color randomizing magnitude
    m_pBrickColorRandomSliderPanel = new SliderPanel("Randomize magnitude:", 0, 500, 150);
    m_pBrickColorRandomSliderPanel.addSliderPanelListener(this);
    pPanel.add(m_pBrickColorRandomSliderPanel, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //4. random mode
    Vector<String> pRandomModeValues = new Vector<String>();
    pRandomModeValues.add("RGB");
    pRandomModeValues.add("Red");
    pRandomModeValues.add("Green");
    pRandomModeValues.add("Blue");
    pRandomModeValues.add("Red - Green");
    pRandomModeValues.add("Red - Blue");
    pRandomModeValues.add("Green - Blue");
    
    pRandomModeValues.add("HSB");
    pRandomModeValues.add("Hue");
    pRandomModeValues.add("Saturation");
    pRandomModeValues.add("Brightness");
    pRandomModeValues.add("Hue - Saturation");
    pRandomModeValues.add("Hue - Brightness");
    pRandomModeValues.add("Saturation - Brightness");
    
    m_pRandomModes = new ComboBoxPanel("Random mode:", pRandomModeValues, 0);
    m_pRandomModes.addComboBoxPanelListener(this);
    pPanel.add(m_pRandomModes, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //4. bevel mode
    Vector<String> pBevelModeValues = new Vector<String>();
    pBevelModeValues.add("None");
    pBevelModeValues.add("Raised");
    pBevelModeValues.add("Sunk");
    m_pBevelModes = new ComboBoxPanel("Bevel mode:", pBevelModeValues, 1);
    m_pBevelModes.addComboBoxPanelListener(this);
    pPanel.add(m_pBevelModes, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //5. draw brick borders
    m_pDrawBrickBorder = new JCheckBox("Draw brick border");
    m_pDrawBrickBorder.setSelected(false);
    m_pDrawBrickBorder.addActionListener(this);
    pPanel.add(m_pDrawBrickBorder, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)

    //6. create the panel for brick border color
    m_pBrickBorderColorDisplay = new ColorDisplay();
    m_pBrickBorderColorDisplay.setColor(Color.BLACK);
    m_pBrickBorderColorDisplay.addColorDisplayListener(this);
    pPanel.add(m_pBrickBorderColorDisplay, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    
    
    
    
    //7. finally add a buffer object
    GridBagLayoutUtilities.addDefaultBufferComponentForPropertyPanels(
      pGridBagConstraints, pPanel
    );
    
    
    //8. return the funky shit
    return pPanel;
    

  }
  */
  
  /*
  private JPanel getSizePanel() {
    
    //1. set the layout
    JPanel pPanel = new JPanel();
    GridBagLayout pGridBag = new GridBagLayout();
    GridBagConstraints pGridBagConstraints = GridBagLayoutUtilities.getDefaultConstraintsForPropertyPanels(null);
    pGridBagConstraints.fill = GridBagConstraints.HORIZONTAL; //the mode how the element fills out its cell
    pPanel.setLayout(pGridBag);
    
    //2. create the panel for brick width
    m_pBrickWidthSliderPanel = new SliderPanel("Brick width:", 0, 500, 20);
    m_pBrickWidthSliderPanel.addSliderPanelListener(this);
    pPanel.add(m_pBrickWidthSliderPanel, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //3. create the panel for brick height
    m_pBrickHeightSliderPanel = new SliderPanel("Brick height:", 0, 500, 10);
    m_pBrickHeightSliderPanel.addSliderPanelListener(this);
    pPanel.add(m_pBrickHeightSliderPanel, pGridBagConstraints);
    pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    //7. finally add a buffer object
    GridBagLayoutUtilities.addDefaultBufferComponentForPropertyPanels(
      pGridBagConstraints, pPanel
    );
    
    
    //8. return the funky shit
    return pPanel;
    

  }
  */
  
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code BrickTextureParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code BrickTextureParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pBrickTexturePrameters;
  }
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code BrickTexturePropertiesPanel} object. 
   *  The set operation is only successful if a {@code BrickTextureParameters} object has been specified.
   *
   *  @param pParameters The {@code BrickTextureParameters} object representing the effect parameters 
   *                     that should be represented by this {@code BrickTexturePropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;
    
    if (pParameters instanceof BrickTextureParameters) {
      m_pBrickTexturePrameters = (BrickTextureParameters)pParameters;
      
      updateControls();
    }
  }
  
  
  
  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the brick dimensions or the randomize magnitude have been adapted in the corresponding slider panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SlimSliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SlimSliderPanel pSliderPanel, int iValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();
    
    //2. set the new parameters
    if (pSliderPanel.equals(m_pBrickColorRandomSliderPanel)) {
      m_pBrickTexturePrameters.setDoubleParameterValue(PARAM_DOUBLE_RANDOMIZE_MAGNITUDE, (double)iValue);
    }
    
    if (pSliderPanel.equals(m_pBrickWidthSliderPanel)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BRICK_WIDTH, iValue);
    }
    
    if (pSliderPanel.equals(m_pBrickHeightSliderPanel)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BRICK_HEIGHT, iValue);
    }
    
    //2. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }
  
  ////////////////////////////////////////
  // COMBOBOX PANEL LISTENER            //
  ////////////////////////////////////////
  /**
   *  If bevel or the randomize mode have been adapted in the corresponding combobox panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pComboBoxPanel The {@code ComboBoxPanel} object that has been modified.
   *  @param iSelectedIndex The selected index of the combo box that has been modified. 
   **/
  public void applyChanges(ComboBoxPanel pComboBoxPanel, int iSelectedIndex) {
  
    //1. make some simple checks
    if (pComboBoxPanel == null) return;
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();
    
    //2. set the new parameters
    if (pComboBoxPanel.equals(m_pBevelModes)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BEVEL_MODE, iSelectedIndex);
    }
    
    if (pComboBoxPanel.equals(m_pRandomModes)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_RANDOM_MODE, iSelectedIndex);
      //System.out.println(iSelectedIndex);
    }
    
    
    
    //3. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }
  
  
  /////////////////////////////////////////
  // ICOLOR DISPLAY LISTENER             //
  /////////////////////////////////////////
  /**
   *  If the color of the brick or the brick border has been changed, this is communicated with this method, 
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that has been changed.
   *  @param pColor The color that has been set for the {@code ColorDisplay} object. 
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();
    
    //1. set the new colors
    m_pBrickTexturePrameters.setColorParameterValue(PARAM_COLOR_BRICK, m_pBrickColorDisplay.getColor());
    m_pBrickTexturePrameters.setColorParameterValue(PARAM_COLOR_BORDER, m_pBrickBorderColorDisplay.getColor());
    
    //2. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }
  
  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  Detects which parameter should be shown now in the properties panel and if the brick border 
   *  should be drawn.
   *
   *  @param pEvent The action event triggering this method. 
   **/
  public void actionPerformed(ActionEvent pEvent) {
    
    //1. property selection
    
      
    
    //1. draw brick border checkbox
    Object pSource = pEvent.getSource();
    if (pSource.equals(m_pDrawBrickBorder)) {
      drawBrickBorderActionPerformed();
      return;
    }
    
    //2. property selection
    propertiesSelectionActionPerformed();
    
  }
  
  
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    populatePropertiesSelection(pLanguageProperties);
    
    
    //4. other gui elements
    m_pDrawBrickBorder.setText(pLanguageProperties.getProperty("BrickTexture.DrawBorder"));
    
    Vector<String> pBevelModeValues = new Vector<String>();
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.None"));
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.Raised"));
    pBevelModeValues.add(pLanguageProperties.getProperty("BrickTexture.BevelMode.Sunk"));
    m_pBevelModes.setValueLabels(pBevelModeValues);
    
    
    Vector<String> pRandomModeValues = new Vector<String>();
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RGB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Red"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Green"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Blue"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RG"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.RB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.GB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HSB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Hue"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Saturation"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.Brightness"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HS"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.HB"));
    pRandomModeValues.add(pLanguageProperties.getProperty("BrickTexture.RandomMode.SB"));
    m_pRandomModes.setValueLabels(pRandomModeValues);
    
    
  
  }
  
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pBrickTexturePrameters == null) return;
    
    //2. TO DO: update all controls according to the settings 
    //   in the effect parameters and make sure that no action is fired!
    m_pBrickColorDisplay.setColor(m_pBrickTexturePrameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }
  
  private void drawBrickBorderActionPerformed() {
    //1. make a simple check
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();
    
    //2. draw border
    boolean bDrawBrickBorder = m_pDrawBrickBorder.isSelected();
    m_pBrickTexturePrameters.setBooleanParameterValue(PARAM_BOOLEAN_DRAW_BORDER, bDrawBrickBorder);
    
    //3. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }
  
  
  private void propertiesSelectionActionPerformed() {
  
    //1. get the selected property
    
    //2. remove the current property panel
    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    
    //3. set the new property panel
    int iSelectedId = getSelectedPropertiesIndex();
    if (iSelectedId == 0) {
      m_pCurrentPropertyPanel = m_pBrickColorDisplay;
    }
    else if (iSelectedId == 1) {
      m_pCurrentPropertyPanel = m_pBrickColorRandomSliderPanel;
    }
    else if (iSelectedId == 2) {
      m_pCurrentPropertyPanel = m_pRandomModes;
    }
    else if (iSelectedId == 3) {
      m_pCurrentPropertyPanel = m_pBevelModes;
    }
    else if (iSelectedId == 4) {
      m_pCurrentPropertyPanel = m_pBrickBorderPanel;
    }
    else if (iSelectedId == 5) {
      m_pCurrentPropertyPanel = m_pBrickWidthSliderPanel;
    }
    else if (iSelectedId == 6) {
      m_pCurrentPropertyPanel = m_pBrickHeightSliderPanel;
    }
   
    
    add(m_pCurrentPropertyPanel);
    
    repaint();
    revalidate();
    
    
  }
  
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties) {
    
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add("BrickTexture.BrickColor");
    pPropertyKeys.add("BrickTexture.BrickColorRandomMagnitude");
    pPropertyKeys.add("BrickTexture.RandomMode");
    pPropertyKeys.add("BrickTexture.BevelMode");
    pPropertyKeys.add("BrickTexture.BrickBorderColor");
    pPropertyKeys.add("BrickTexture.BrickWidth");
    pPropertyKeys.add("BrickTexture.BrickHeight");
    populatePropertiesSelection(pLanguageProperties, pPropertyKeys);
    
    
  }
  
  
  
}