package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *   This interface should be implemented by all classes that listen to the current progress or the
 *   end of the execution of a thread.
 *
 *
 **/
public interface IThreadListener {


  /**
   *   This method is called by the thread to tell all its listeners what it is currently processing and
   *   and which percentage of the work has already been finished.
   *
   *   @param dProgress    The percentage of the work that has already been finished.
   *   @param pCurrentWork A <code>String</code> explaning what the thread currently processes.
   *                       This <code>String</code> can also be <code>null</code>.
   **/
  public void setProgressStatus(double dProgress, String pCurrentWork);

  /**
   *   This method is called by the thread, after it has been finished properly.
   **/
  public void endedProperly();

  /**
   *   This method is called by the thread, if it was interrupted and could not be finished properly.
   *   A <code>String</code> representation of the reason, why the thread has been interrupted is given as
   *   parameter.
   *
   *   @param pReason The reason, why the thread has been interrupted as <code>String</code>.
   **/
  public void interrupted(String pReason);



}
