package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  Represents an image or video effect parameter that has a an arbitrary object as value. 
 **/

public class ObjectParameter implements IParameter {



  /*** member variables ***/
  private Object m_pParameterValue;
  private Class m_pClass;



  /////////////////////////
  // CONSTRUCTOR         //
  /////////////////////////
  /**
   *  Creates an {@code ObjectParameter} object with the specified parameter as value. 
   *
   * @param pValue The value object that should be represented by this {@code ObjectParameter} object.
   **/
  public ObjectParameter(Object pValue) {
    this(pValue, null);
  }
  
  
  /**
   *  Creates an {@code ObjectParameter} object with the specified parameter as value. 
   *
   * @param pValue The value object that should be represented by this {@code ObjectParameter} object.
   * @param pClass The type of the object. This can be null. 
   **/
  public ObjectParameter(Object pValue, Class pClass) {

    //1. set the member variables
    m_pParameterValue = pValue;
    m_pClass = pClass;
    if ((pClass == null) && (pValue != null)) m_pClass = pValue.getClass();
  }



  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /**
   * Sets the value that should be represented by this {@code ObjectParameter} object. 
   * The object that is specified must be of the correct type with which this {@code ObjectParameter} has been initiated.
   *
   * @param pValue The value that should be represented by this {@code ObjectParameter} object.
   *
   * @return true, if the object value could be set. If the type of the object is not correct, false is returned and the value is not set.
   **/   
  public boolean setValue(Object pValue) {
    //1. try to cast the object, to check if it is a valid type
    if (m_pClass != null) {
      try {
        m_pClass.cast(pValue);
      }
      catch (Exception pException) {
        return false;
      }
    }
    
    //2.  set the value
    m_pParameterValue = pValue;
    return true;

  }
  
  
  /**
   * Not implemented. TO DO: Create a functionality that allows to create objects from strings. (Is this really necessary?)
   *
   * @param pValue The value that should be represented by this {@code ObjectParameter} object.
   **/
  public boolean setValue(String pValue) {
    
    return false;
  }


  /////////////////////////
  // GET METHODS         //
  /////////////////////////
  /**
   * Returns the value that is represented by this {@code ObjectParameter} object.
   *
   * @return The value that is represented by this {@code ObjectParameter} object.
   **/   
  public Object getValue() {
    return m_pParameterValue;
  }


  


}
