package util.image;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java imports
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.util.Vector;

/**
 * Provides methods to combine several images side by side. TO DO: Get rid of static methods and use object oriented style?
 **/
public class ImageCombineOperations {

  /**
   *    Returns an image that shows all of the specified images. The images are placed in 2 columns. 
   *
   *    @param pImages A list of images that should be shown on one image.
   *
   *    @return A {@code BufferedImage} object that shows all the specified images in a 2 column layout.
   **/
  public static BufferedImage concatenateImages(Vector<BufferedImage> pImages) {

    //1. make some checks
    if (pImages == null) return null;
    if (pImages.size() == 0) return null;

    //2. identify the real image number
    //System.out.println("2");
    int iImageNum = pImages.size();

    int iCounter = 0;
    for (int i = 0; i < iImageNum; i++) {
      if (pImages.get(i) != null) iCounter++;
    }
    if (iCounter == 0) return null;

    int iRealImageNum = iCounter;
    boolean bEvenImageNum = ((iImageNum % 2) == 0);


    //3. identify the width and height of the result image
    //System.out.println("3");
    int iWidth = 0;
    int iHeight = 0;
    for (int i = 0; i < iImageNum; i++) {
      BufferedImage pImage = pImages.get(i);

      if (pImage != null) {

        if (iWidth < pImage.getWidth()) iWidth = pImage.getWidth();
        if (iHeight < pImage.getHeight()) iHeight = pImage.getHeight();

      }
    }

    int iOverallWidth, iOverallHeight;

    if (iRealImageNum > 1) {
      iOverallWidth = 2* iWidth;
    }
    else {
      iOverallWidth = iWidth;
    }

    iOverallHeight = iHeight * (iRealImageNum/2);
    if (!bEvenImageNum) iOverallHeight += iHeight;


    //4. create the new image
    BufferedImage pConcatenatedImage = new BufferedImage(iOverallWidth, iOverallHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pConcatenatedImage.createGraphics();

    //5. draw the images
    iCounter = 0;
    for (int i = 0; i < iImageNum; i++) {
      BufferedImage pImage = pImages.get(i);

      if (pImage != null) {
        int iXPos = (iCounter % 2) * iWidth;
        int iYPos = (iCounter / 2) * iHeight;

        //System.out.println(iXPos + ", " + iYPos);

        pGraphics.drawImage(pImage, iXPos, iYPos, null);

        iCounter++;
      }
    }


    return pConcatenatedImage;






  }

}